#!/usr/bin/env python

# This is the convusb program, inputs hex character stream (like what's
# dumped by Kai Wang's krepdump.ko kernel module, which has a 4 place hex
# address field, then a single space, then groups of 2 digit bytes) and
# puts out parsed HID code, followed by the hex bytes that drove that line.
# Flag=  -f argument, which changes the input from stdin to whatever comes after
# the -f option 
#
# Thanks go to both Kai Wang and Markus Brueffer, two denizens of the FreeBSD-USB
# mailing list who went WAY out of their way to be of help to me in learning
# what I think is the worst spec I have ever bumped up against.
# Chuck Robey (chuckr@chuckr.org).
# Get the latest version at http://people.freebsd.org/~chuckr/code/python/convusb.py-1.0

import sys, os, os.path, getopt, string, socket, Tkinter, textwrap
import items, parse, tables


class TKgui(object):
    displayIndex = 0
    store = None
    dispPrev = None
    dispAddr = None
    dispInputLine1 = None
    dispInputLine2 = None
    dispInputLine3 = None
    dispInputLine4 = None
    dispbTag = None
    dispbType = None
    dispbSize = None
    dispClass = None
    dispName = None
    dispDataLength = None
    dispDataValue = None
    dispDataDecimal = None
    dispDataText = None
    dispUnitExponent = None
    dispUnitSystem = None
    dispUnits = None
    dispHelpListing = None
    po = None
    io = None
    tw = None

    def __init__(self, io, po):
        # paint the GUI, accepts a dict which has been stuffed with
        # all the parsed data lines, because this GUI allows scrolling
        # to any item in the storehouse of data.

        self.io = io
        self.po = po
        self.store = self.io.itemStore
        self.tw = textwrap.TextWrapper(initial_indent="* ",width=64)


        # First part, make root
        self.cgRoot = Tkinter.Tk()

        # part 2, establish all graphic variables

        self.dispPrev = Tkinter.StringVar()
        self.dispAddr = Tkinter.StringVar()
        self.dispInputLine1 = Tkinter.StringVar()
        self.dispInputLine2 = Tkinter.StringVar()
        self.dispInputLine3 = Tkinter.StringVar()
        self.dispInputLine4 = Tkinter.StringVar()
        self.dispbTag = Tkinter.StringVar()
        self.dispbType = Tkinter.StringVar()
        self.dispbSize = Tkinter.StringVar()
        self.dispClass = Tkinter.StringVar()
        self.dispName = Tkinter.StringVar()
        self.dispDataLength = Tkinter.StringVar()
        self.dispDataValue = Tkinter.StringVar()
        self.dispDataDecimal = Tkinter.StringVar()
        self.dispDataText = Tkinter.StringVar()
        self.dispUnitExponent = Tkinter.StringVar()
        self.dispUnitSystem = Tkinter.StringVar()
        self.dispUnits = Tkinter.StringVar()
        self.dispCollectionTypes = Tkinter.StringVar()
        self.dispUsageTypes = Tkinter.StringVar()
        self.dispHelpListing = Tkinter.StringVar()

        # all other parts follow this form: new frame, packit, then add fields
        self.cgFrame1 = Tkinter.Frame(self.cgRoot)
        self.cgFrame1.pack(side = 'top')
        self.cgTitle = Tkinter.Label(self.cgFrame1,text = """Chuckr's USB HID Parser""",width = 40, pady=25, padx = 5, fg='blue',bg='tan', font = ('Times', 22), relief=Tkinter.RIDGE)
        self.cgTitle.pack(side = 'left', fill=Tkinter.X)
        Tkinter.Frame(self.cgRoot, height = 4, pady = 4, bg = 'blue',relief = Tkinter.RAISED, bd = 4).pack(anchor = 'w', side = 'top', expand = 1, fill = Tkinter.X)

        # Label row 1
        self.cgFrame2 = Tkinter.Frame(self.cgRoot, pady = 4)
        self.cgFrame2.pack(anchor = 'w', side = 'top')
        self.cgPrev = Tkinter.Label(self.cgFrame2, text = 'PrevLnCarryOver', width = 15, font = ('Helvetica', 16),)
        self.cgPrev.pack(side = 'left', anchor = 'w')
        self.cgAddr = Tkinter.Label(self.cgFrame2, text = 'Addr', width = 4, font = ('Helvetica', 18))
        self.cgAddr.pack(side = 'left', padx = 4)
        self.cgInputLn = Tkinter.Label(self.cgFrame2, text = '(----------------------InputLine(Hex)----------------------)', width = 50, font = ('Helvetica', 16))
        self.cgInputLn.pack(side = 'left', anchor = 'w')

        # Output row 1
        self.cgFrame3 = Tkinter.Frame(self.cgRoot)
        self.cgFrame3.pack(anchor = 'w', side = 'top')
        pad0 =  Tkinter.Label(self.cgFrame3, width = 5)
        pad0.pack(side='left')
        self.cgDispPrev = Tkinter.Label(self.cgFrame3, width = 4, textvariable = self.dispPrev, fg = "black", bg='lightgreen',  font = ('Helvetica', 18), relief = Tkinter.RAISED)
        self.cgDispPrev.pack(side='left')
        pad1 =  Tkinter.Label(self.cgFrame3, width = 7)
        pad1.pack(side='left')
        self.cgDispAddr = Tkinter.Label(self.cgFrame3, width = 4, textvariable = self.dispAddr, fg = "black", bg='lightgreen',  font = ('Helvetica', 18), relief = Tkinter.RAISED)
        self.cgDispAddr.pack(side='left',padx = 24)
        pad2 =  Tkinter.Label(self.cgFrame3, width = 3)
        pad2.pack(side='left')
        self.cgInputLine1 = Tkinter.Label(self.cgFrame3, textvariable = self.dispInputLine1, fg = "black", bg='white',  font = ('Helvetica', 18), relief = Tkinter.FLAT)
        self.cgInputLine1.pack(side='left')
        self.cgInputLine2 = Tkinter.Label(self.cgFrame3, textvariable = self.dispInputLine2, fg = "red", bg='lightblue',  font = ('Helvetica', 18), relief = Tkinter.FLAT)
        self.cgInputLine2.pack(side='left')
        self.cgInputLine3 = Tkinter.Label(self.cgFrame3, textvariable = self.dispInputLine3, fg = "red", bg='yellow',  font = ('Helvetica', 18), relief = Tkinter.FLAT)
        self.cgInputLine3.pack(side='left')
        self.cgInputLine4 = Tkinter.Label(self.cgFrame3, textvariable = self.dispInputLine4, fg = "black", bg='white',  font = ('Helvetica', 18), relief = Tkinter.FLAT)
        self.cgInputLine4.pack(side='left')
        Tkinter.Frame(self.cgRoot, height = 4, pady = 4, bg = 'blue',relief = Tkinter.RAISED, bd = 4).pack(anchor = 'w', side = 'top', expand = 1, fill = Tkinter.X)


        # Label row 2
        self.cgFrame4 =  Tkinter.Frame(self.cgRoot)
        self.cgFrame4.pack(anchor = 'w', side = 'top')
        self.cgbTagLabel = Tkinter.Label(self.cgFrame4, width = 4, text = 'bTag', font = ('Helvetica', 14, 'bold'))
        self.cgbTagLabel.pack(side = 'left')

        self.cgbTypeLabel = Tkinter.Label(self.cgFrame4, width = 4, padx = 4, text  = 'bType', font = ('Helvetica', 14))
        self.cgbTypeLabel.pack(side = 'left', padx = 4)

        self.cgbSizeLabel = Tkinter.Label(self.cgFrame4, width = 5, text  = 'bSize', font = ('Helvetica', 14, 'bold'))
        self.cgbSizeLabel.pack(side = 'left')
        self.cgitemClass = Tkinter.Label(self.cgFrame4, width = 8, text = 'Class', font = ('Helvetica', 14))
        self.cgitemClass.pack(side = 'left')
        self.cgitemName = Tkinter.Label(self.cgFrame4, width =31, text = 'Name', font = ('Helvetica', 16))
        self.cgitemName.pack(side = 'left')
        Tkinter.Label(self.cgFrame4, width = 8).pack(side='left')
        self.cgdataLength = Tkinter.Label(self.cgFrame4, width = 6, text = 'Length', font = ('Helvetica', 16)) 
        self.cgdataLength.pack(side = 'left')
        Tkinter.Label(self.cgFrame4, width = 3).pack(side='left')
        self.cgdataValue = Tkinter.Label(self.cgFrame4, width = 8, text = 'Value', font = ('Helvetica', 16)) 
        self.cgdataValue.pack(side = 'left', padx= 15)
        Tkinter.Label(self.cgFrame4, width = 3).pack(side='left')
        self.cgdataDecimal = Tkinter.Label(self.cgFrame4, width = 8, text = '(base10)', font = ('Helvetica', 16))
        self.cgdataDecimal.pack(side = 'left')

        # Output row 2
        self.cgFrame5 =  Tkinter.Frame(self.cgRoot)
        self.cgFrame5.pack(anchor = 'w', side = 'top',pady=8)
        self.cgdispbTag = Tkinter.Label(self.cgFrame5, width = 2, textvariable = self.dispbTag, fg = "black", bg='lightblue', font = ('Helvetica', 18), relief = Tkinter.RAISED)
        self.cgdispbTag.pack(side = 'left', padx = 8)
        self.cgdispbType = Tkinter.Label(self.cgFrame5, width = 1, textvariable = self.dispbType, fg = "black", bg='lightgreen', font = ('Helvetica', 18), relief = Tkinter.RAISED)
        self.cgdispbType.pack(side = 'left', padx = 20)
        self.cgdispbSize = Tkinter.Label(self.cgFrame5, width = 1, textvariable = self.dispbSize, fg = "black", bg='lightblue', font = ('Helvetica', 18), relief = Tkinter.RAISED)
        self.cgdispbSize.pack(side = 'left', padx = 8)
        self.cgdispClass = Tkinter.Label(self.cgFrame5, width = 8, textvariable = self.dispClass, fg = "black", bg='lightgreen', font = ('Helvetica', 18), relief = Tkinter.RAISED)
        self.cgdispClass.pack(side = 'left', padx = 4)
        self.cgdispName = Tkinter.Label(self.cgFrame5, width = 24, textvariable = self.dispName, fg = "black", bg='lightblue', font = ('Helvetica', 18), relief = Tkinter.RAISED)
        self.cgdispName.pack(side = 'left', padx = 18)
        self.cgdispDataLength = Tkinter.Label(self.cgFrame5, width = 10, textvariable = self.dispDataLength, fg = "black", bg='lightgreen', font = ('Helvetica', 18), relief = Tkinter.RAISED)
        self.cgdispDataLength.pack(side = 'left', padx = 4)
        self.cgdispDataValue = Tkinter.Label(self.cgFrame5, width = 8, textvariable = self.dispDataValue, fg = "black", bg='lightblue', font = ('Helvetica', 18), relief = Tkinter.RAISED)
        self.cgdispDataValue.pack(side = 'left', padx = 4)
        self.cgdispDataDecimal = Tkinter.Label(self.cgFrame5, width = 8, textvariable = self.dispDataDecimal, fg = "black", bg='lightblue', font = ('Helvetica', 18), relief = Tkinter.RAISED)
        self.cgdispDataDecimal.pack(side = 'left', padx = 14, expand = 1, fill = Tkinter.X)
        Tkinter.Frame(self.cgRoot, height = 4, pady = 4, bg = 'blue',relief = Tkinter.RAISED, bd = 4).pack(anchor = 'w', side = 'top', expand = 1, fill = Tkinter.X)

        # Label row 3
        self.cgFrame6 =  Tkinter.Frame(self.cgRoot)
        self.cgFrame6.pack(anchor = 'sw', side = 'top')
        self.cgCollectionTypes = Tkinter.Label(self.cgFrame6, anchor = 'sw', padx = 18, width = 36, fg = 'black', text = 'Collection Types', font = ('Helvetica', 16))
        self.cgCollectionTypes.pack(anchor = 'sw', side = 'left')
        Tkinter.Label(self.cgFrame6, width = 12).pack(side='left')
        self.cgUsageTypes = Tkinter.Label(self.cgFrame6, anchor = 'sw',width = 36, fg = 'black', text = 'Usage Types', font = ('Helvetica', 16))
        self.cgUsageTypes.pack(anchor = 'sw', side = 'left')

        # output row 3
        self.cgFrame7 =  Tkinter.Frame(self.cgRoot)
        self.cgFrame7.pack(anchor = 'nw', side = 'top',pady=8)
        self.cgdispCollectionTypes = Tkinter.Label(self.cgFrame7,anchor = 'nw',  width = 36, textvariable = self.dispCollectionTypes, padx = 4, fg = "black", bg='lightblue', font = ('Helvetica', 18), relief = Tkinter.RAISED)
        self.cgdispCollectionTypes.pack(side = 'left', anchor = 'nw', padx = 8)
        self.cgdispUsageTypes = Tkinter.Label(self.cgFrame7, width = 36,anchor = 'nw',  textvariable = self.dispUsageTypes, padx = 4, fg = "black", bg='lightblue', font = ('Helvetica', 18), relief = Tkinter.RAISED)
        self.cgdispUsageTypes.pack(side = 'left',anchor = 'nw',  padx = 8)
        Tkinter.Frame(self.cgRoot, height = 4, pady = 4, bg = 'blue',relief = Tkinter.RAISED, bd = 4).pack(anchor = 'w', side = 'top', expand = 1, fill = Tkinter.X)
        
        # Label row 4
        self.cgFrame8 =  Tkinter.Frame(self.cgRoot)
        self.cgFrame8.pack(anchor = 'w', side = 'top', pady = 12)
        self.cgunitSystem = Tkinter.Label(self.cgFrame8, anchor = 'w',width = 18, text = '        Unit System', font = ('Helvetica', 16))
        self.cgunitSystem.pack(anchor = 'w', side = 'left')
        Tkinter.Label(self.cgFrame8, width = 3).pack(side='left')
        self.cgunitExponent = Tkinter.Label(self.cgFrame8, anchor = 'w',width = 13, text = 'Data Exponent', font = ('Helvetica', 16))
        self.cgunitExponent.pack(anchor = 'w', side = 'left')
        self.cgunits = Tkinter.Label(self.cgFrame8, anchor = 'w',width = 18, padx = 44, text = 'Units ^ exponents', font = ('Helvetica', 16))
        self.cgunits.pack(anchor = 'w', side = 'left')

        # Output row 4
        self.cgFrame9 =  Tkinter.Frame(self.cgRoot)
        self.cgFrame9.pack(anchor = 'w', side = 'top', pady = 0)
        self.cgdispUnitSystem = Tkinter.Label(self.cgFrame9, width = 16, textvariable = self.dispUnitSystem, padx = 4, fg = "black", bg='lightblue', font = ('Helvetica', 18), relief = Tkinter.RAISED)
        self.cgdispUnitSystem.pack(side = 'left', padx = 8)
        self.cgdispUnitExponent = Tkinter.Label(self.cgFrame9, width = 2, textvariable = self.dispUnitExponent, padx = 40, fg = "black", bg='lightblue', font = ('Helvetica', 18), relief = Tkinter.RAISED)
        self.cgdispUnitExponent.pack(side = 'left', padx = 8)
        cgdispUnit = Tkinter.Label(self.cgFrame9, width = 32, textvariable = self.dispUnits, padx = 40, fg = "black", bg='lightblue', font = ('Helvetica', 18), relief = Tkinter.RAISED)
        cgdispUnit.pack(side = 'left', padx = 8)
        Tkinter.Frame(self.cgRoot, height = 4, pady = 4, bg = 'blue',relief = Tkinter.RAISED, bd = 4).pack(anchor = 'w', side = 'top', expand = 1, fill = Tkinter.X)

        # Label row 5
        self.cgFrame10 = Tkinter.Frame(self.cgRoot,pady = 8)
        self.cgFrame10.pack(anchor = 'sw', side = 'top')
        self.cgunit = Tkinter.Label(self.cgFrame10, anchor = 'w',width = 22, text = 'Input/Output/\nFeature Notations', font = ('Helvetica', 16))
        self.cgunit.pack(anchor = 'sw', side = 'left')
        self.cghelptop = Tkinter.Label(self.cgFrame10, anchor = 'w',width = 12, text = 'Help\nTopics', font = ('Helvetica', 16))
        self.cghelptop.pack(anchor = 'sw', side = 'left')
        pad3 = Tkinter.Label(self.cgFrame10, anchor = 'w', width = 6, font = ('Helvetica', 18))
        pad3.pack(anchor = 'sw', side = 'left')
        self.cghelpLabel1 = Tkinter.Label(self.cgFrame10, anchor = 'w',width = 20, text = 'Help Explanations, Hit', font = ('Helvetica', 16))
        self.cghelpLabel2 = Tkinter.Label(self.cgFrame10, anchor = 'w',width = 6, text = 'Button', font = ('Helvetica', 16),fg='red')
        self.cghelpLabel3 = Tkinter.Label(self.cgFrame10, anchor = 'w',width = 10, text = ' to update', font = ('Helvetica', 16))
        self.cghelpLabel1.pack(anchor = 'sw', side = 'left')
        self.cghelpLabel2.pack(anchor = 'sw', side = 'left')
        self.cghelpLabel3.pack(anchor = 'sw', side = 'left')


        # Output row 5
        self.cgFrame11 = Tkinter.Frame(self.cgRoot)
        self.cgFrame11.pack(anchor = 'w', side = 'top')

        # scrolled list for IOFQualifiers
        self.cgUnitScr = Tkinter.Scrollbar(self.cgFrame11)
        self.cgUnitScr.pack(side=Tkinter.LEFT, fill=Tkinter.Y)
        self.cgIOFQ = Tkinter.Listbox(self.cgFrame11,relief=Tkinter.RAISED, width = 14,font = ('Helvetica', 16))
        self.cgIOFQ.pack(anchor = 'w', side=Tkinter.LEFT, fill=Tkinter.Y)
        # bind listbox to scrollbar
        self.cgIOFQ.config(yscrollcommand=self.cgUnitScr.set)
        self.cgUnitScr.config(command=self.cgIOFQ.yview)


        # Help scrolled list for data field explanations
        self.cgHelpScr = Tkinter.Scrollbar(self.cgFrame11)
        self.cgHelpScr.pack(side=Tkinter.LEFT, fill=Tkinter.Y)
        self.cgHelp = Tkinter.Listbox(self.cgFrame11,relief=Tkinter.RAISED, width = 14,font = ('Helvetica', 16), selectmode = Tkinter.SINGLE)
        self.cgHelp.pack(anchor = 'w', side=Tkinter.LEFT, fill=Tkinter.Y)
        # bind listbox to scrollbar
        self.cgHelp.config(yscrollcommand=self.cgHelpScr.set)
        self.cgHelpScr.config(command=self.cgHelp.yview)

        self.cghelpListing = Tkinter.Label(self.cgFrame11, height = 12, width = 62, fg = 'navyblue', bg = 'pink', textvariable = self.dispHelpListing, relief = Tkinter.RAISED, font = ('Helvetica', 16), anchor = 'w')
        self.cghelpListing.pack(side = 'right', anchor = 'w')
        Tkinter.Frame(self.cgRoot, height = 4, pady = 4, bg = 'blue',relief = Tkinter.RAISED, bd = 4).pack(anchor = 'w', side = 'top', expand = 1, fill = Tkinter.X)


        # Button row, final, for Next, Prev, Help, and Quit
        self.cgFrame10 = Tkinter.Frame(self.cgRoot)
        self.cgFrame10.pack(anchor = 'w', side = 'top', pady = 18)

        self.cgfwdbutton = Tkinter.Button(self.cgFrame10, fg = 'blue', bg = 'green', bd = 4, command = self.fwdButton, relief = Tkinter.SUNKEN, text = 'Next', font = ('Helvetica', 14))
        self.cgfwdbutton.pack(anchor = 'w', side = 'left', expand = 1, fill = Tkinter.X)
        self.cgbackbutton = Tkinter.Button(self.cgFrame10, text = 'Prev', command=self.backButton, relief = Tkinter.SUNKEN, bg = 'green', fg = 'blue', bd = 4, font = ('Helvetica', 14))
        self.cgbackbutton.pack(anchor = 'w', side = 'left')
        self.cgkillbutton = Tkinter.Button(self.cgFrame10, text = 'Quit', command=self.killButton, relief = Tkinter.SUNKEN, bg = 'green', fg = 'blue', bd = 4, font = ('Helvetica', 14))
        self.cgkillbutton.pack(anchor = 'e', side = 'right')
        self.cgHelpButton = Tkinter.Button(self.cgFrame10, text = 'Help Topic Button', command=self.helpButton, relief = Tkinter.SUNKEN, bg = 'green', fg = 'red', bd = 4, font = ('Helvetica', 14))
        self.cgHelpButton.pack(anchor = 'e', side = 'right')

        self.display()
        # The help data is totally static, so list all topics right now!
        for help in self.io.tab.field.keys():
            self.cgHelp.insert(Tkinter.END, help)

        self.cgRoot.bind("<q>", self.exitButton)
        self.cgRoot.mainloop()

    def backButton(self):
        if self.displayIndex > 0:
            self.displayIndex = self.displayIndex - 1
            self.display()

    def fwdButton(self):
        if self.displayIndex < len(self.store):
            self.displayIndex = self.displayIndex + 1
            self.display()

    def helpButton(self):
        t = self.tw.fill(self.io.tab.field[self.cgHelp.get(self.cgHelp.curselection())])
        self.dispHelpListing.set(t)

    def exitButton(self, char):
        sys.exit(0)

    def killButton(self):
        sys.exit(0)

    def rF(self, name):
        if name in self.store[self.displayIndex].keys():
            return self.store[self.displayIndex][name]
        else:
            return ''

    def display(self):
        """This displays for regular Labels easily, but the display for
        scrolled lists is a mite more complicated (need a loop), see below."""
        self.dispPrev.set(self.rF('Prev'))
        self.dispAddr.set(self.rF('Addr'))
        self.dispInputLine1.set(self.rF('InputLineBegin'))
        self.dispInputLine2.set(self.rF('InputLineCommand'))
        self.dispInputLine3.set(self.rF('InputLineData'))
        self.dispInputLine4.set(self.rF('InputLineEnding'))
        self.dispbTag.set(self.rF('bTag'))
        self.dispbType.set(self.rF('bType'))
        self.dispbSize.set(self.rF('bSize'))
        self.dispName.set(self.rF('Name'))
        self.dispClass.set(self.rF('Class'))
        self.dispDataValue.set(self.rF('DataValue'))
        self.dispDataLength.set(self.rF('DataLength'))
        self.dispDataDecimal.set(self.rF('DecimalData'))
        self.dispUnits.set(self.rF('Units'))
        self.dispUnitSystem.set(self.rF('UnitSystem'))
        self.dispCollectionTypes.set(self.rF('CollectionTypes'))
        # If it's a Usage, stick the UsagePage in before it with a separating '/'
        if ('Usage' in self.store[self.displayIndex]) and ('UsagePage' in self.store[self.displayIndex]):
            self.dispUsageTypes.set(self.rF('UsagePage') + '/' + self.rF('Usage'))
        else:
            self.dispUsageTypes.set(self.rF('UsagePage'))
        self.dispUnitExponent.set(self.rF('UnitExponent'))
        if 'IOFQualifiers' in self.store[self.displayIndex].keys():
            dlist = self.store[self.displayIndex]['IOFQualifiers'].split(',')
            for d in dlist:
                self.cgIOFQ.insert(Tkinter.END, d)
        else:
            self.cgIOFQ.delete(0, Tkinter.END)

if __name__ == '__main__':

    sys.exit(0)

