#!/usr/bin/env python

# This is the convusb program, inputs hex character stream (like what's
# dumped by Kai Wang's krepdump.ko kernel module, which has a 4 place hex
# address field, then a single space, then groups of 2 digit bytes) and
# puts out parsed HID code, followed by the hex bytes that drove that line.
# Flag=  -f argument, which changes the input from stdin to whatever comes after
# the -f option 
#
# Thanks go to both Kai Wang and Markus Brueffer, two denizens of the FreeBSD-USB
# mailing list who went WAY out of their way to be of help to me in learning
# what I think is the worst spec I have ever bumped up against.
# Chuck Robey (chuckr@chuckr.org).
# Get the latest version at http://people.freebsd.org/~chuckr/code/python/convusb.py-1.0

import sys, socket, copy, tables

class items(object):
    """This class takes the packaged items (list of hex-ascii chars) from the parse
    object, and wiht it's internal tables, stores the fully identified items in the
    items.output array, which can be accessed via the items.getParsedItem() member"""
    tab = tables.tables()
    storedItems = []
    cur = []
    tdict = {}
    itemStore = []
    currentUsagePageTable = None
    currentOutputIndex = 0
    addr = []
    usedLine = []
    currentLine = []
    po = None


    def storeCurrentData(self):
        """AFTER all of the data for the current entry has been entered via setCurrentData,
        this member stores that (now fully charged) dictionary into the itemStore list, which
        is a list of dictionaries.  Make sure to store a deep copy of tdict, not tdict itself."""
        # can't just store tdict, because we're going to erase tdict so we can reuse it, so dup
        # tdict in a temp var, and store that.
        self.itemStore = self.itemStore + [self.tdict.copy()]
        self.tdict.clear()

    
    def setCurrentData(self, name, item):
        """This knows the name of EVERY piece of data, else it tosses an error.
        It stores that passed  in data (sans
        name) in a global place, self.itemStore[], each item of which is a
        dictionary.  Some things, like Units, are always lists, even if they are len(1),
        but that's ok, it's expected.  If a new list items is being stored, just add
        it to any present value (will always be a list)"""
        ml = 0
        if name == 'Prev':
            self.tdict['Prev']=item;
            return;
        elif name == 'Addr':
            self.tdict['Addr'] = item;
            return;
        elif name == 'InputLineBegin':
            self.tdict['InputLineBegin']=item;
            return;
        elif name == 'InputLineCommand':
            self.tdict['InputLineCommand']=item;
            return;
        elif name == 'InputLineData':
            self.tdict['InputLineData']=item;
            return;
        elif name == 'InputLineEnding':
            self.tdict['InputLineEnding']=item;
            return;
        elif name == 'bTag':
            self.tdict['bTag']=item;
            return;
        elif name == 'bType':
            self.tdict['bType']=item;
            return;
        elif name == 'bSize':
            self.tdict['bSize']=item;
            return;
        elif name == 'Class':
            self.tdict['Class']=item;
            return;
        elif name == 'Name':
            self.tdict['Name']=item;
            return;
        elif name == 'DataValue':
            self.tdict['DataValue']=item;
            return;
        elif name == 'DataLength':
            self.tdict['DataLength']=item;
            return;
        elif name == 'DecimalData':
            self.tdict['DecimalData']=item;
            return;
        elif name == 'UnitExponent':
            self.tdict['UnitExponent']=item;
            return;
        elif name == 'UnitSystem':
            self.tdict['UnitSystem']=item;
            return;
        elif name == 'Units':
            # This one comes in bits, so arrange for each new entry to add to existing one
            if isinstance(item,list) == False:
                item = [item]
            if 'Units' in self.tdict.keys():
                self.tdict['Units'] = self.tdict['Units'] + item
            else:
                self.tdict['Units'] = item;
            return;
        elif name == 'UsagePage':
            self.tdict['UsagePage']=item;
            return;
        elif name == 'Usage':
            self.tdict['Usage']=item;
            return;
        elif name == 'CollectionTypes':
            self.tdict['CollectionTypes']=item;
            return;
        elif name == 'IOFQualifiers':
            # This one comes in bits, so arrange for each new entry to add to existing one
            if 'IOFQualifiers' in self.tdict.keys():
                self.tdict['IOFQualifiers'] = self.tdict['IOFQualifiers'] + ',' + item
            else:
                self.tdict['IOFQualifiers'] = item;
            return;
        else:
            # Unrecognized input error
            print >> sys.stderr, 'In items.StoreCurrentData, unrecognized data type %s data value %s' % (name, item)
            sys.exit(1)
    
    def flip(self, num):
        if len(num) < 2:
            return ''
        if len(num) == 2:
            return num
        if len(num) == 4:
            return num[2:] + num[0:2]
        if len(num) == 8:
            return num[6:] + num[4:6] + num[2:4] + num[0:2]

    def getData(self):
        if len(self.cur[2:]) == 0:
            return ''
        if len(self.cur[2:]) == 2:
            return ( '%02X' % (int(self.flip(self.cur[2:]),16)))
        elif len(self.cur[2:]) == 4:
            return ( '%04X' % (int(self.flip(self.cur[2:]),16)))
        elif len(self.cur[2:]) == 8:
            return ( '%08X' % (int(self.flip(self.cur[2:]),16)))

    def getbTag(self):
        """Top 4 bits of command byte, since there are 4 bits/char, use top char."""
        return int(self.cur[0],16)

    def getbType(self):
        """Two top bits of 2nd char, coded, 0->main, 1->global, 2->local, 3->reserved."""
        return (int(self.cur[1],16) & 0xC) >> 2

    def getbSize(self):
        """Bottom 2 bits of first byte, 0=0, 01=1, 02=2, 03=4 bytes data size."""
        if len(self.cur) < 2:
            return int(100);
        if int(self.cur[1],16) & 0x3 == 0: # (1 + 0) * 2 = 2
            return int(2)
        if int(self.cur[1],16) & 0x3 == 1: # (1 + 1) * 2 = 4
            return int(4)
        if int(self.cur[1],16) & 0x3 == 2: # (1 + 2) * 2 = 6
            return int(6)
        if int(self.cur[1],16) & 0x3 == 3: # (1 + 4) * 2 = 10
            return int(10)

    def dispatchItems(self):
        self.setCurrentData('Prev',self.po.prev)
        self.setCurrentData('InputLineBegin',self.po.usedLine)
        self.setCurrentData('InputLineCommand',self.cur[0:2])
        self.setCurrentData('InputLineData',self.data)
        self.setCurrentData('InputLineEnding',self.po.currentLine)
        self.setCurrentData('Addr',self.po.addr)
        self.setCurrentData('bType',self.bType)
        self.setCurrentData('bTag',self.bTag)
        self.setCurrentData('bSize',self.bSize)
        if len(self.data) > 0:
            self.setCurrentData('DataValue',self.data)
            self.setCurrentData('DataLength',('%d' % len(self.getData())))
            self.setCurrentData('DecimalData',('%d' % int(self.data,16)))
        if self.bType == 0: # main
            self.setCurrentData('Class','Main')
            self.parseMain()
        elif self.bType == 1: # global
            self.setCurrentData('Class','Global')
            self.parseGlobal()
        elif self.bType == 2: # local
            self.setCurrentData('Class','Local')
            self.parseLocal()
        else:
            print 'Impossible bType value %s' % bType
            sys.exit(1)
        self.storeCurrentData()
        return;

    def parseMain(self):
        if self.bTag == 0x0C:
            self.setCurrentData('Name', 'End Collection')
            return
        elif self.bTag in (0x08, 0x09, 0x0B): # input, output, and feature fields have subfields
            mdata = self.tab.getMainCodes(self.bTag)  # data tuple, so do decode
            mval = 0x0001
            mtext = ''
            while mval <= 0x100:
                if ((mval & int(self.data,16)) != 0) and mdata[mval][1] != 'Reserved':
                    self.setCurrentData('IOFQualifiers', mdata[mval][1])
                elif mdata[mval][0] != 'Reserved':
                    self.setCurrentData('IOFQualifiers', mdata[mval][0])
                mval = mval << 1;
            self.setCurrentData('Name', self.tab.getMainItems(self.bTag))
            return
        elif self.bTag == 0x0A:  # collection item
            mdata = self.tab.getMainCodes(self.bTag)  # data tuple, so do decode
            self.setCurrentData('Name', 'Collection')
            self.setCurrentData('CollectionTypes', mdata[int(self.data,16)])
            return
        else:
            print 'parseMain, undefined main key value %s cur %s' % (self.bTag, self.cur)

    def parseGlobal(self):
        if self.bType != 1:
            print "in parseGlobal, not a Global class item, bType %s self.cur=%s" % (self.bType, self.cur)
            return
        if len(self.cur) != self.bSize:
            print "in parseGlobal, wrong length, len(self.cur)=%d bSize=%d" % (len(self.cur), self.bSize)
            return 'Illegal'
        if self.bTag not in self.tab.globalItems.keys():
            print 'parseGlobal, Fatal error, looking up bad global lookup key value %s' % self.bTag
            return 'Illegal'
        # Need to handle units and unit exponents as special cases, AND realize that
        # the parsing on these two is best described as goofy.
        if self.bTag == 5: # unit exponent
            self.setCurrentData('Name', 'Unit Exponent')
            self.setCurrentData('UnitExponent', self.tab.exponents[int(self.getData(),16)])
        if self.bTag == 6: # Unit, worse POS I have ever seen
            self.setCurrentData('Name', 'UNIT')
            unitColumn = (int(self.data,16) & 0x0f)
            self.setCurrentData('UnitSystem',self.tab.unitSystem[unitColumn])
            needComma = False
            rangeValue = len(self.data)
            for ind in range(1, rangeValue):
                exponent = ((int(self.data,16) & (0x0f >> (ind * 4))) + 1)
                if ((int(self.data,16) >> (ind * 4)) & 0xF) > 0:
                    self.setCurrentData('Units',self.tab.unit[unitColumn][(ind)] + '^^' + ('%d' % (((int(self.data,16) >> (ind * 4)) & 0xF))))
            #return
        if self.bTag==0 and len(self.data)>0: # Usage Page, look up usages
            edata = self.tab.usages[int(self.data,16)][0]
            self.setCurrentData('UsagePage', self.tab.usages[int(self.data,16)][0])
            # store the name of the usage page table, because that's a Local type item, and
            # we'll need this page when the Local/Usage comes on by.
            self.currentUsagePageTable = self.tab.usages[int(self.data,16)]
            #return 
        parse = self.tab.globalItems[self.bTag]
        self.setCurrentData('Name', parse)
        return


    def parseLocal(self):
        if self.bType != 2:
            print "in parseLocal, not a Local class item, bType %s self.cur=%s" % (self.bType, self.cur)
            sys.exit(1)
        if len(self.cur) != self.bSize:
            print "in parseLocal, wrong length, len(self.cur)=%d bSize=%d" % (len(self.cur), self.bSize)
            sys.exit(1)
        if self.bTag not in self.tab.localItems.keys():
            return "Undefined item"
        self.setCurrentData('Name',self.tab.localItems[self.bTag])
        if self.bTag == 0: # 0 means usage, get the page value
            if len(self.data)==8:
                # take Usage Page from top half of data
                if int(self.cur[2:4],16) not in self.tab.usages.keys():
                    return
                self.upage = self.tab.usages[int(self.data,16)][1]
            elif self.currentUsagePageTable != None:
                self.upage = self.currentUsagePageTable
            else:
                # if no usage page, die
                return
            if len(self.upage) == 0 or int(self.data,16) == 0:
                return
            if len(self.data) > 2:
                self.setCurrentData('Name', self.tab.localItems[self.bTag])
                self.setCurrentData('UsagePage', self.upage[1][(int(self.data,16) & 0xFF00)>>8])
                self.setCurrentData('Usage', self.upage[1][int(self.data,16) & 0xFF])
                return
            else:
                self.setCurrentData('Name', self.tab.localItems[self.bTag])
                self.setCurrentData('UsagePage', self.upage[0])
                self.setCurrentData('Usage', self.upage[1][int(self.data,16)])
                return
        return

    def __init__(self, parseObject):
        self.po = parseObject
        i = self.po.getItem()
        while i != []:
            (self.po.addr, self.cur, self.po.usedLine, self.po.currentLine, self.po.prev) = i
            if len(self.cur) == 0:
                sys.exit(0)
            self.bTag   = self.getbTag()
            self.bType  = self.getbType()
            self.bSize  = self.getbSize()
            self.data   = self.getData()
            self.dispatchItems()
            i = self.po.getItem()
