#!/usr/bin/env python

# This is the convusb program, inputs hex character stream (like what's
# dumped by Kai Wang's krepdump.ko kernel module, which has a 4 place hex
# address field, then a single space, then groups of 2 digit bytes) and
# puts out parsed HID code, followed by the hex bytes that drove that line.
# Flag=  -f argument, which changes the input from stdin to whatever comes after
# the -f option 
#
# Thanks go to both Kai Wang and Markus Brueffer, two denizens of the FreeBSD-USB
# mailing list who went WAY out of their way to be of help to me in learning
# what I think is the worst spec I have ever bumped up against.
# Chuck Robey (chuckr@chuckr.org).
# Get the latest version at http://people.freebsd.org/~chuckr/code/python/convusb.py-1.0

import sys, os, os.path, getopt, socket, copy

class parse(object):
    """Passed an input file object, which was chosen in the main module according to input flags.
    This class uses it's built-in knowledge of how long each item will be to forward out only
    the next item's hex chars for parsing.  This has a static idea of the input files lines."""

    readItems = []
    currentLine = None
    usedLine = ''
    previousItem = []
    addr = []
    prev = '    '
    item = ''

    def __init__(self, infile):
        self.readItems = infile.readlines()
        infile.close()
        self.popLine()

    def popLine(self):
        if len(self.readItems) == 0:
            self.addr = ''
            self.currrentLine = ''
        elif len(self.readItems) == 1:
            self.addr = (self.readItems[0].split(' ',1))[0].upper()[-4:]
            self.currentLine = ''
            for b in self.readItems[0].split(' ',1)[1].split(' '):
                self.currentLine = self.currentLine + b.upper().rstrip()
            self.readItems = []
        else:
            self.addr = (self.readItems[0].split(' ',1))[0].upper()[-4:]
            self.currentLine = ''
            for b in self.readItems[0].split(' ',1)[1].split(' '):
                self.currentLine = self.currentLine + b.upper().rstrip()
            self.readItems = self.readItems[1:]
        return

    def getItem(self):
        """This is the way that items get out of the parse object, and into the items object.
        This has to pass a 3 part tuple, so the items object can see line address, pre-item
        and post-item data, as well as the current data."""
        self.usedLine = self.usedLine + self.item
        if len(self.currentLine) < 1:
            if len(self.readItems) < 1:
                return []
            self.popLine()
            self.usedLine = ''
            self.prev = '    '
        self.item = ''
        il = ((self.getItemLength() + 1) * 2)
        ic = len(self.currentLine)
        if il > ic:
            self.prev = self.currentLine
        while  len(self.item) < il:
            if len(self.currentLine) < 1:
                self.popLine()
                self.usedLine = ''
            self.item = self.item + self.currentLine[0:2]
            self.currentLine = self.currentLine[2:]
        retval = (self.addr, self.item, copy.deepcopy(self.usedLine), copy.deepcopy(self.currentLine),self.prev)
        return retval

    def getItemLength(self):
        if len(self.currentLine) > 1:
            return(int(self.currentLine[1],16) & 0x03)
        else:
            return 0
    
        sys.exit(0)

if __name__ == '__main__':

    # for test purposes only


    inputFileFlag = False # -f
    useGUI = False;
    parsedDataStore = {}

    o, a = getopt.getopt(sys.argv[1:], 'hgf:')
    opts = {}
    for k,v in o:
        opts[k] = v
    if  opts.has_key('-h'):
        usage(); sys.exit(0)
    if  opts.has_key('-g'):
        useGUI = True;
    if opts.has_key('-f'):
        inputFileFlag = opts['-f']


    infile = open(inputFileFlag, 'r')
    new = parse(infile)
    it = new.getItem()
    while it != []:
        it = new.getItem()

    sys.exit(0)

